(function ($) {
  'use strict';

  Drupal.Cloud = {};

  const colors = [
    ['#edf8e9', '#bae4b3', '#74c476', '#31a354', '#006d2c'],
    ['#e9f8f4', '#b3e4dc', '#74c1c4', '#317fa3', '#003f6d'],
  ];

  const AXIS_HEIGHT = 10;
  const AXIS_FONT_SIZE = '8';
  const CHART_HEIGHT = 20;
  const TITLE_FONT_SIZE = 10;
  const TITLE_POSITION_X = 1;
  const TITLE_POSITION_Y = 70;
  const BACKGROUND_DEFAULT_COLOR = 'rgba(0, 0, 0, 0)';

  // Draw Chart Function.
  Drupal.Cloud.drawChart = function (data, first) {
    data.forEach(function (item, idx) {
      // Delete current chart.
      d3.select(item.selector).select('svg').remove();

      // Draw charts.
      if (first && idx === 0) {
        // Delete the axis.
        d3.select('#x-axis').remove();

        // Add <div> for the axis.
        $(item.selector).before('<div id="x-axis"></div>');

        // Draw charts.
        const axis = new HorizonChart();
        axis.setHeight(AXIS_HEIGHT);
        axis.setDataSet(item.values);
        axis.setSelector('#x-axis');
        axis.draw();

        const chartg = d3.selectAll('#x-axis g:first-child');
        chartg.style('display', 'none');
        const axisg = d3.selectAll('#x-axis .axis');
        axisg.attr('font-size', AXIS_FONT_SIZE);
      }

      // Draw charts.
      const horizonChart = new HorizonChart();
      const fontColor = $(item.selector).css('color');

      let bgColor = '';
      $(item.selector).parents().each(function () {
        console.log(getComputedStyle(this).backgroundColor);
        let color = getComputedStyle(this).backgroundColor;
        if (color && color !== BACKGROUND_DEFAULT_COLOR) {
          bgColor = color;
          return false;
        }
      });
      horizonChart.setTitle(item.id);
      horizonChart.setTitleFontSize(TITLE_FONT_SIZE);
      horizonChart.setTitlePosition(TITLE_POSITION_X, TITLE_POSITION_Y);
      horizonChart.setTitleColer(fontColor);
      horizonChart.setHeight(CHART_HEIGHT);
      horizonChart.setDataSet(item.values);
      horizonChart.setSelector(item.selector);
      horizonChart.setColors(colors[idx % 2]);
      horizonChart.setShowXScale(false);
      horizonChart.draw();

      const g = d3.selectAll(item.selector + ' svg g');
      const title = d3.selectAll(item.selector + ' svg text');
      const bbox = $(item.selector + ' svg text')[0].getBBox();

      g.insert('rect', item.selector + ' svg text')
      .attr('x', bbox.x - 1)
      .attr('y', bbox.y)
      .attr('width', bbox.width + 2)
      .attr('height', bbox.height)
      .attr('fill', bgColor)
      .style('opacity', '0.8');

      $(item.selector).css('margin-bottom', '0');

      if (!first || idx > 0) {
        $(item.selector).css('border-top', '1px solid #ddd');
      }
    });
  };
})(jQuery);
