Introduction
============

- Cloud Dashboard enables Single Page Application (SPA).

Requirements
============

- Drupal `9.4.x` or higher (The latest version of Drupal `9.x`)
- Cloud `5.x` or higher
- [Simple OAuth](
    https://www.drupal.org/project/simple_oauth
  ) `5.0`.x module (Not `5.2.x` or higher)
- JSON:API module

Installation
============

- `composer require drupal/cloud`
- For other tasks, please check [INSTALL.md](INSTALL.md).

Build from source code
============

- If you want to build the source code for the SPA-side, please check
  [BUILD.md](BUILD.md).

NPM libraries used in this module
============

- react v17
  - The UI library by JavaScript/TypeScript.
- react-router-dom v5
  - The routing library for React.
- react-bootstrap
  - The CSS/JS library for React.
  - Unlike normal web development, `bootstrap` is supplied by the Rigel theme.
- leaflet, react-leaflet
  - The chart librart by JavaScript.
  - react-leaflet is a wrapper library for leaflet.js in React.
- d3
  - The 2D graphic library by JavaScript.
  - Used to graphically display server metrics.
- i18next, react-i18next
  - A library to support multilingual display of web applications.
  - The definition of words to be translated is described in i18n.ts.
- react-icons
  - The dataset of icons for React.
- react-loader-spinner
  - The spinner logo library for React.

Directory structure
============

- cloud_dashboard
  - src
    - atoms, molecules, organisms, pages
      - Various *.tsx files according to Atomic design.
    - constant
      - others
        - Definition files for various constants.
      - templates
        - Template files for describing various forms.
    - hooks
      - drupal_jsonapi.ts
        - Wrapper library for using JSON:API.
      - drupal_oauth2.ts
        - Wrapper library for using OAuth2.
    - model
      - Various files of data models.
    - service
      - convert.ts
        - Service class for converting data for transmission or display.
      - state.ts
        - Service class for managing cloud context.
      - storage.ts
        - Service class for editing localstorage.
      - string.ts
        - Service class for string manipulation.
    - i18n.ts
      - The definition of words to be translated.
  - build.sh
    - The shell script to build forms written in React.
  - package.json
    - Definition of library used in forms written in React.
- css
  - Put the CSS generated by building code written in React.
- js
  - Put the JavaScript generated by building code written in React.
- src
  - Controller/CloudDashboardConfigController.php
    - The definition of REST APIs.
  - Form
    - Config/CloudDashboardAdminSettings.php
      - The config form of this custom module.
    - React/CloudDashboardForm.php
      - The base form for React.
- cloud_dashboard.info.yml
- cloud_dashboard.libraries.yml
  - Mapping CSS/JS generated by building code for React to Drupal forms.
- cloud_dashboard.links.menu.yml
- cloud_dashboard.module
  - Implements hook_help().
- cloud_dashboard.routing.yml
  - The routing of the base form for React.
  - The routing of the configuration form of this custom module.
  - The routing of REST APIs used by this custom module.
