import EntityFormTemplate from 'model/EntityFormTemplate';

const OPENSTACK_PORT_TEMPLATE: EntityFormTemplate[] = [
  {
    cloudServiceProvider: 'openstack',
    entityName: 'port',
    actionType: 'create',
    entityRecords: [
      {
        type: 'panel',
        panelName: 'Port',
        keyValueRecords: [
          { type: 'default', labelName: 'Name', name: 'name', defaultValue: '', required: true },
          {
            type: 'select', labelName: 'Network', name: 'network_id',
            url: '/cloud_dashboard/openstack/{cloud_context}/networks',
            defaultValue: '',
            required: true,
          },
          { type: 'default', labelName: 'Device ID', name: 'device_id', defaultValue: '' },
          { type: 'default', labelName: 'Device owner', name: 'device_owner', defaultValue: '' },
          { type: 'select-local', name: 'ip_address_or_subnet', labelName: 'Specify IP address or subnet', defaultValue: '', required: true, value: [
            { name: 'unspecified', labelName: 'Unspecified' },
            { name: 'subnet', labelName: 'Subnet' },
            { name: 'fixed_ip', labelName: 'Fixed IP address' },
          ] },
          {
            type: 'select', labelName: 'Subnet', name: 'subnet',
            url: '/cloud_dashboard/openstack/{cloud_context}/subnets/{network_id}',
            defaultValue: '',
          },
          { type: 'default', labelName: 'Fixed IP address', name: 'fixed_ips', defaultValue: '' },
          { type: 'default', labelName: 'MAC address', name: 'mac_address', defaultValue: '' },
          {
            type: 'boolean', labelName: 'Port security', name: 'port_security_enabled',
            defaultValue: true
          },
          {
            type: 'multi-select', labelName: 'Security group', name: 'security_groups',
            url: '/cloud_dashboard/openstack/{cloud_context}/security_groups',
            defaultValue: [],
          },
          { type: 'select-local', name: 'binding_vnic_type', labelName: 'VNIC type', defaultValue: '', required: true, value: [
            { name: 'normal', labelName: 'Normal' },
            { name: 'direct', labelName: 'Direct' },
            { name: 'macvtap', labelName: 'MacVTap' },
            { name: 'baremetal', labelName: 'Bare Metal' },
            { name: 'virtio-forwarder', labelName: 'Virtio Forwarder' },
          ] },
        ]
      }
    ]
  },
  {
    cloudServiceProvider: 'openstack',
    entityName: 'port',
    actionType: 'edit',
    entityRecords: [
      {
        type: 'panel',
        panelName: 'Port',
        keyValueRecords: [
          { type: 'default', labelName: 'Name', name: 'name', defaultValue: '', required: true },
          {
            type: 'boolean', labelName: 'Enable admin state', name: 'admin_state_up',
            defaultValue: false
          },
          {
            type: 'join', labelName: 'Network', name: 'network_id', info: {
              entityTypeId: 'openstack_network',
              keyColumn: 'network_id',
            }, defaultValue: '', readOnly: true
          },
          { type: 'select-local', name: 'binding_vnic_type', labelName: 'VNIC type', defaultValue: '', required: true, value: [
            { name: 'normal', labelName: 'Normal' },
            { name: 'direct', labelName: 'Direct' },
            { name: 'macvtap', labelName: 'MacVTap' },
            { name: 'baremetal', labelName: 'Bare Metal' },
            { name: 'virtio-forwarder', labelName: 'Virtio Forwarder' },
          ] },
          {
            type: 'boolean', labelName: 'Port security', name: 'port_security_enabled',
            defaultValue: true
          },
          {
            type: 'multi-select', labelName: 'Security group', name: 'security_groups',
            url: '/cloud_dashboard/openstack/{cloud_context}/security_groups',
            defaultValue: [],
          },
          { type: 'item-array', labelName: 'Allowed Address Pairs', name: 'allowed_address_pairs', defaultValue: [], info: [
            { labelName: 'IP address or CIDR', name: 'item_key', type: 'default' },
            { labelName: 'MAC address', name: 'item_value', type: 'default' },
          ] },
        ]
      }
    ]
  },
  {
    cloudServiceProvider: 'openstack',
    entityName: 'port',
    actionType: 'delete',
    entityRecords: [
      {
        type: 'label',
        text: 'Are you sure you want to delete the {{entityName}} {{name}}?'
      },
    ],
    submitButtonLabel: 'Delete'
  },
]

export default OPENSTACK_PORT_TEMPLATE;
