import EntityFormTemplate from 'model/EntityFormTemplate';

const OPENSTACK_SUBNET_TEMPLATE: EntityFormTemplate[] = [
  {
    cloudServiceProvider: 'openstack',
    entityName: 'subnet',
    actionType: 'create',
    entityRecords: [
      {
        type: 'panel',
        panelName: 'Subnet',
        keyValueRecords: [
          { type: 'default', labelName: 'Name', name: 'name', defaultValue: '', required: true },
          {
            type: 'select', labelName: 'Network', name: 'network_id',
            url: '/cloud_dashboard/openstack/{cloud_context}/networks',
            defaultValue: '',
            required: true,
          },
          { type: 'default', labelName: 'CIDR', name: 'cidr', defaultValue: '', required: true },
          { type: 'select-local', name: 'ip_version', labelName: 'IP version', defaultValue: '', required: true, value: [
            { name: 'IPv4', labelName: 'IPv4' },
            { name: 'IPv6', labelName: 'IPv6' },
          ] },
          { type: 'default', labelName: 'Gateway IP', name: 'gateway_ip', defaultValue: '' },
          {
            type: 'boolean', labelName: 'Disable Gateway', name: 'disable_gateway',
            defaultValue: false
          },
          {
            type: 'boolean', labelName: 'Enable DHCP', name: 'enable_dhcp',
            defaultValue: true
          },
          {
            type: 'textarea', labelName: 'Allocation pools', name: 'allocation_pools',
            defaultValue: ''
          },
          {
            type: 'textarea', labelName: 'DNS Name Servers', name: 'dns_name_servers',
            defaultValue: ''
          },
          {
            type: 'textarea', labelName: 'Host Routes', name: 'host_routes',
            defaultValue: ''
          },
        ]
      }
    ]
  },
  {
    cloudServiceProvider: 'openstack',
    entityName: 'subnet',
    actionType: 'edit',
    entityRecords: [
      {
        type: 'panel',
        panelName: 'Subnet',
        keyValueRecords: [
          { type: 'default', labelName: 'Name', name: 'name', defaultValue: '', required: true },
          {
            type: 'join', labelName: 'Network', name: 'network_id', info: {
              entityTypeId: 'openstack_network',
              keyColumn: 'network_id',
            }, defaultValue: '', readOnly: true
          },
          { type: 'default', labelName: 'CIDR', name: 'cidr', defaultValue: '', required: true },
          { type: 'default', labelName: 'Gateway IP', name: 'gateway_ip', defaultValue: '' },
          {
            type: 'boolean', labelName: 'Disable Gateway', name: 'disable_gateway',
            defaultValue: false
          },
          {
            type: 'boolean', labelName: 'Enable DHCP', name: 'enable_dhcp',
            defaultValue: true
          },
          {
            type: 'textarea', labelName: 'Allocation pools', name: 'allocation_pools',
            defaultValue: ''
          },
          {
            type: 'textarea', labelName: 'DNS Name Servers', name: 'dns_name_servers',
            defaultValue: ''
          },
          {
            type: 'textarea', labelName: 'Host Routes', name: 'host_routes',
            defaultValue: ''
          },
        ]
      }
    ]
  },
  {
    cloudServiceProvider: 'openstack',
    entityName: 'subnet',
    actionType: 'delete',
    entityRecords: [
      {
        type: 'label',
        text: 'Are you sure you want to delete the {{entityName}} {{name}}?'
      },
    ],
    submitButtonLabel: 'Delete'
  },
]

export default OPENSTACK_SUBNET_TEMPLATE;
