import { CLOUD_SERVICE_PROVIDER_LIST, DEFAULT_CLOUD_CONTEXTS } from 'constant/others/other';
import useDrupalJsonApi from 'hooks/drupal_jsonapi';
import CloudContext from 'model/CloudContext';
import CloudServiceProvider from 'model/CloudServiceProvider';
import { createContext, useEffect, useState } from 'react';

interface AppState {
  cloudContextList: CloudContext[];
  loadingFlg: boolean;
}

export const useAppState = (): AppState => {
  const { getEntityListAll } = useDrupalJsonApi();
  const [cloudContextList, setCloudContextList] = useState<CloudContext[]>([...DEFAULT_CLOUD_CONTEXTS]);
  const [loadingFlg, setLoadingFlg] = useState(true);

  // Set cloud context list.
  useEffect(() => {
    const init = async () => {
      let newCloudContextList = [...DEFAULT_CLOUD_CONTEXTS];
      for (const cloudServiceProvider of CLOUD_SERVICE_PROVIDER_LIST) {
        try {
          const data = (await getEntityListAll('cloud_config', {}, cloudServiceProvider))
            .map((record: any) => {
              return {
                cloudServiceProvider: cloudServiceProvider as CloudServiceProvider,
                name: record.attributes.cloud_context,
                labelName: record.attributes.name
              };
            });
          newCloudContextList = [...newCloudContextList, ...data];
        } catch {
          // pass
        }
      }
      setCloudContextList(newCloudContextList);
      setLoadingFlg(false);
    };
    init();
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);

  return {
    cloudContextList,
    loadingFlg
  };
};

export const AppContext = createContext<AppState>({
  cloudContextList: [],
  loadingFlg: true,
});
