<?php

namespace Drupal\cloud_dashboard\Form\React;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class CloudReactForm.
 *
 * @package Drupal\cloud_dashboard\Form\React
 */
class CloudDashboardForm extends FormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  private $entityTypeManager;

  /**
   * The cloud service provider plugin manager.
   *
   * @var \Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface
   */
  private $cloudConfigPluginManager;

  /**
   * Constructs new CloudLocalTasks.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface $cloud_config_plugin_manager
   *   The cloud service provider plugin manager.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    CloudConfigPluginManagerInterface $cloud_config_plugin_manager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->cloudConfigPluginManager = $cloud_config_plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.cloud_config_plugin')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cloud_dashboard';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['html'] = [
      '#markup' => '<div id="root"></div>',
      '#attached' => [
        'library' => 'cloud_dashboard/react_app',
      ],
    ];

    // Get cloud configs.
    $cloud_configs = $this->entityTypeManager->getStorage('cloud_config')->loadMultiple();

    $bundles = [];
    foreach ($cloud_configs ?: [] as $cloud_config) {
      if (array_key_exists($cloud_config->bundle(), $bundles)) {
        continue;
      }
      $this->cloudConfigPluginManager->setCloudContext($cloud_config->getCloudContext());
      $route = $this->cloudConfigPluginManager->getInstanceCollectionTemplateName();

      $matches = [];
      preg_match("/view\.{$cloud_config->bundle()}_(\w+)\.list/", $route, $matches);

      $bundles[$cloud_config->bundle()] = [
        'entityTypeId' => count($matches) > 1 ? $matches[1] : '',
        'label' => $cloud_config->type->entity->label(),
      ];
    }

    $form['#attached']['drupalSettings']['cloud_configs'] = array_keys($bundles);
    $form['#attached']['drupalSettings']['cloud_config_infos'] = $bundles;

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
  }

}
