(function ($) {
  'use strict';

  // The interval to update a view's content.
  let refresh_in_sec = drupalSettings.aws_cloud_view_refresh_interval || 10;

  // The function to get a view's content in an Ajax way.
  let auto_update = function () {

    // The URL of the callback for a view.
    let url = $('div.views-form form').attr('action');
    let query_str = '';
    let pos = url.indexOf('?');
    if (pos !== -1) {
      query_str = url.substr(pos);
      url = url.substr(0, pos);
    }
    url += '/callback';

    // Update .view-content element, which is the content of view, excluding
    // exposed filter and pager.
    $.get(url + query_str, function (data) {
      let form_token = $('.view-content input[name=form_token]').val();
      let checkbox_ids = $('.view-content .form-checkbox:checked').map(function () {
        return this.id;
      }).get();

      // Replace table.
      $('.views-element-container .view-content div.table-responsive')
        .replaceWith($(data).find('.view-content div.table-responsive'));

      // Replace pager.
      $('.views-element-container nav.pager-nav')
        .replaceWith($(data).find('nav.pager-nav'));

      // Restore form token.
      // If form token changes, the form validation will fail.
      if (form_token) {
        $('.view-content input[name=form_token]').val(form_token);
      }

      // Fix for the theme Bartik, which need to initialize drop buttons.
      if (Drupal.behaviors.dropButton) {
        Drupal.behaviors.dropButton.attach(document, drupalSettings);
      }

      if (Drupal.behaviors.tableSelect) {
        Drupal.behaviors.tableSelect.attach(document, drupalSettings);
      }

      // Fix button initialization for bootstrap theme.
      if (Drupal.behaviors.bootstrapDropdown) {
        Drupal.behaviors.bootstrapDropdown.attach(document, drupalSettings);
      }

      // Restore checkbox selection.
      $.each(checkbox_ids, function (i, val) {
        if (val) {
          $('#' + val).prop('checked', true);
        }
        else {
          $('.select-all .form-checkbox').prop('checked', true);
        }
      });
    });
  };

  // Update a view's content every "refresh_in_sec" seconds.
  setInterval(auto_update, refresh_in_sec * 1000);
})(jQuery);
