<?php

namespace Drupal\cloud\Form;

use Drupal\Core\Form\FormStateInterface;

/**
 * Form controller for cloud store edit forms.
 *
 * @ingroup cloud_Project
 */
class CloudStoreForm extends CloudContentForm {

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    /** @var \Drupal\cloud\Entity\CloudStore $entity */
    $entity = $this->entity;
    $cloud_store_type = $entity->bundle();
    $form = parent::buildForm($form, $form_state);

    $form = $this->reorderFieldset($form, $cloud_store_type);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state): void {
    $entity = $this->entity;

    // Save entity.
    if ($entity->save()) {

      $this->processOperationStatus($entity, 'created');

      $form_state->setRedirect('entity.cloud_store.collection', [
        'cloud_store_type' => $entity->bundle(),
      ]);
    }
    else {
      $this->processOperationErrorStatus($entity, 'created');
    }

    // Clear block and menu cache.
    $this->clearCacheValues($entity->getCacheTags());
  }

  /**
   * Add the build array of fieldset others.
   *
   * @param array &$form
   *   The form array.
   * @param string $cloud_store_type
   *   Cloud store type.
   *
   * @return array
   *   The formulated form array.
   */
  protected function reorderFieldset(array &$form, $cloud_store_type = ''): array {
    $entity_type = $this->entity->getEntityType()->id();
    $field_definitions = \Drupal::service('entity_field.manager')->getFieldDefinitions($entity_type, $cloud_store_type);
    $fields = [];
    foreach ($field_definitions ?: [] as $key => $value) {
      if (preg_match('/field_(\w+)/', $key)) {
        $fields[] = $key;
      }
    }

    $fieldsets_def = [
      [
        'name' => 'store',
        'title' => $this->t('Store name'),
        'open' => TRUE,
        'fields' => [
          'name',
          'cloud_context',
        ],
      ],
      [
        'name' => 'parameter',
        'title' => $this->t('Parameter'),
        'open' => TRUE,
        'fields' => $fields,
      ],
      [
        'name' => 'others',
        'title' => $this->t('Others'),
        'open' => FALSE,
        'fields' => [
          'revision_log_message',
          'uid',
        ],
      ],
    ];

    \Drupal::service('cloud')->reorderForm($form, $fieldsets_def);
    return $form;
  }

}
